@@ -6,11 +6,12 @@ import sys
 from difflib import get_close_matches
 from pathlib import Path
 from types import SimpleNamespace
-from typing import Dict, Union
+from typing import Dict, List, Union
 
-from ultralytics import __version__, yolo
-from ultralytics.yolo.utils import (DEFAULT_CFG_DICT, DEFAULT_CFG_PATH, LOGGER, PREFIX, USER_CONFIG_DIR,
-                                    IterableSimpleNamespace, checks, colorstr, yaml_load, yaml_print)
+from ultralytics import __version__
+from ultralytics.yolo.utils import (DEFAULT_CFG, DEFAULT_CFG_DICT, DEFAULT_CFG_PATH, LOGGER, PREFIX, ROOT,
+                                    USER_CONFIG_DIR, IterableSimpleNamespace, colorstr, emojis, yaml_load, yaml_print)
+from ultralytics.yolo.utils.checks import check_yolo
 
 CLI_HELP_MSG = \
     """
@@ -27,7 +28,7 @@ CLI_HELP_MSG = \
         yolo detect train data=coco128.yaml model=yolov8n.pt epochs=10 lr0=0.01
 
     2. Predict a YouTube video using a pretrained segmentation model at image size 320:
-        yolo segment predict model=yolov8n-seg.pt source=https://youtu.be/Zgi9g1ksQHc imgsz=320
+        yolo segment predict model=yolov8n-seg.pt source='https://youtu.be/Zgi9g1ksQHc' imgsz=320
 
     3. Val a pretrained detection model at batch-size 1 and image size 640:
         yolo detect val model=yolov8n.pt data=coco128.yaml batch=1 imgsz=640
@@ -68,7 +69,7 @@ def cfg2dict(cfg):
     return cfg
 
 
-def get_cfg(cfg: Union[str, Path, Dict, SimpleNamespace], overrides: Dict = None):
+def get_cfg(cfg: Union[str, Path, Dict, SimpleNamespace] = DEFAULT_CFG, overrides: Dict = None):
     """
     Load and merge configuration data from a file or dictionary.
 
@@ -87,6 +88,11 @@ def get_cfg(cfg: Union[str, Path, Dict, SimpleNamespace], overrides: Dict = None
         check_cfg_mismatch(cfg, overrides)
         cfg = {**cfg, **overrides}  # merge cfg and overrides dicts (prefer overrides)
 
+    # Type checks
+    for k in 'project', 'name':
+        if isinstance(cfg[k], (int, float)):
+            cfg[k] = str(cfg[k])
+
     # Return instance
     return IterableSimpleNamespace(**cfg)
 
@@ -111,6 +117,33 @@ def check_cfg_mismatch(base: Dict, custom: Dict):
         sys.exit()
 
 
+def merge_equals_args(args: List[str]) -> List[str]:
+    """
+    Merges arguments around isolated '=' args in a list of strings.
+    The function considers cases where the first argument ends with '=' or the second starts with '=',
+    as well as when the middle one is an equals sign.
+
+    Args:
+        args (List[str]): A list of strings where each element is an argument.
+
+    Returns:
+        List[str]: A list of strings where the arguments around isolated '=' are merged.
+    """
+    new_args = []
+    for i, arg in enumerate(args):
+        if arg == '=' and 0 < i < len(args) - 1:  # merge ['arg', '=', 'val']
+            new_args[-1] += f"={args[i + 1]}"
+            del args[i + 1]
+        elif arg.endswith('=') and i < len(args) - 1 and '=' not in args[i + 1]:  # merge ['arg=', 'val']
+            new_args.append(f"{arg}{args[i + 1]}")
+            del args[i + 1]
+        elif arg.startswith('=') and i > 0:  # merge ['arg', '=val']
+            new_args[-1] += arg
+        else:
+            new_args.append(arg)
+    return new_args
+
+
 def argument_error(arg):
     return SyntaxError(f"'{arg}' is not a valid YOLO argument.\n{CLI_HELP_MSG}")
 
@@ -130,7 +163,7 @@ def entrypoint(debug=False):
     It uses the package's default cfg and initializes it using the passed overrides.
     Then it calls the CLI function with the composed cfg
     """
-    args = ['train', 'predict', 'model=yolov8n.pt'] if debug else sys.argv[1:]
+    args = ['train', 'model=yolov8n.pt', 'data=coco128.yaml', 'imgsz=32', 'epochs=1'] if debug else sys.argv[1:]
     if not args:  # no arguments passed
         LOGGER.info(CLI_HELP_MSG)
         return
@@ -139,18 +172,18 @@ def entrypoint(debug=False):
     modes = 'train', 'val', 'predict', 'export'
     special = {
         'help': lambda: LOGGER.info(CLI_HELP_MSG),
-        'checks': checks.check_yolo,
+        'checks': check_yolo,
         'version': lambda: LOGGER.info(__version__),
         'settings': lambda: yaml_print(USER_CONFIG_DIR / 'settings.yaml'),
         'cfg': lambda: yaml_print(DEFAULT_CFG_PATH),
         'copy-cfg': copy_default_config}
 
     overrides = {}  # basic overrides, i.e. imgsz=320
-    for a in args:
+    for a in merge_equals_args(args):  # merge spaces around '=' sign
         if '=' in a:
             try:
                 re.sub(r' *= *', '=', a)  # remove spaces around equals sign
-                k, v = a.split('=')
+                k, v = a.split('=', 1)  # split on first '=' sign
                 if k == 'cfg':  # custom.yaml passed
                     LOGGER.info(f"{PREFIX}Overriding {DEFAULT_CFG_PATH} with {v}")
                     overrides = {k: val for k, val in yaml_load(v).items() if k != 'cfg'}
@@ -183,23 +216,53 @@ def entrypoint(debug=False):
         else:
             raise argument_error(a)
 
-    cfg = get_cfg(DEFAULT_CFG_DICT, overrides)  # create CFG instance
+    # Defaults
+    task2model = dict(detect='yolov8n.pt', segment='yolov8n-seg.pt', classify='yolov8n-cls.pt')
+    task2data = dict(detect='coco128.yaml', segment='coco128-seg.yaml', classify='mnist160')
 
-    # Mapping from task to module
-    module = {"detect": yolo.v8.detect, "segment": yolo.v8.segment, "classify": yolo.v8.classify}.get(cfg.task)
-    if not module:
-        raise SyntaxError(f"yolo task={cfg.task} is invalid. Valid tasks are: {', '.join(tasks)}\n{CLI_HELP_MSG}")
+    # Mode
+    mode = overrides['mode']
+    if mode is None:
+        mode = DEFAULT_CFG.mode or 'predict'
+        LOGGER.warning(f"WARNING ⚠️ 'mode=' is missing. Valid modes are {modes}. Using default 'mode={mode}'.")
+    elif mode not in modes:
+        if mode != 'checks':
+            raise ValueError(emojis(f"ERROR ❌ Invalid 'mode={mode}'. Valid modes are {modes}."))
+        LOGGER.warning("WARNING ⚠️ 'yolo mode=checks' is deprecated. Use 'yolo checks' instead.")
+        check_yolo()
+        return
 
-    # Mapping from mode to function
-    func = {
-        "train": module.train,
-        "val": module.val,
-        "predict": module.predict,
-        "export": yolo.engine.exporter.export}.get(cfg.mode)
-    if not func:
-        raise SyntaxError(f"yolo mode={cfg.mode} is invalid. Valid modes are: {', '.join(modes)}\n{CLI_HELP_MSG}")
+    # Model
+    model = overrides.pop('model', DEFAULT_CFG.model)
+    task = overrides.pop('task', None)
+    if model is None:
+        model = task2model.get(task, 'yolov8n.pt')
+        LOGGER.warning(f"WARNING ⚠️ 'model=' is missing. Using default 'model={model}'.")
+    from ultralytics.yolo.engine.model import YOLO
+    overrides['model'] = model
+    model = YOLO(model)
 
-    func(cfg)
+    # Task
+    if task and task != model.task:
+        LOGGER.warning(f"WARNING ⚠️ 'task={task}' conflicts with {model.task} model {overrides['model']}. "
+                       f"Inheriting 'task={model.task}' from {overrides['model']} and ignoring 'task={task}'.")
+    task = model.task
+    overrides['task'] = task
+    if mode == 'predict' and 'source' not in overrides:
+        overrides['source'] = DEFAULT_CFG.source or ROOT / "assets" if (ROOT / "assets").exists() \
+            else "https://ultralytics.com/images/bus.jpg"
+        LOGGER.warning(f"WARNING ⚠️ 'source=' is missing. Using default 'source={overrides['source']}'.")
+    elif mode in ('train', 'val'):
+        if 'data' not in overrides:
+            overrides['data'] = task2data.get(task, DEFAULT_CFG.data)
+            LOGGER.warning(f"WARNING ⚠️ 'data=' is missing. Using {model.task} default 'data={overrides['data']}'.")
+    elif mode == 'export':
+        if 'format' not in overrides:
+            overrides['format'] = DEFAULT_CFG.format or 'torchscript'
+            LOGGER.warning(f"WARNING ⚠️ 'format=' is missing. Using default 'format={overrides['format']}'.")
+
+    # Run command in python
+    getattr(model, mode)(**overrides)
 
 
 # Special modes --------------------------------------------------------------------------------------------------------
