@@ -3,12 +3,13 @@
 from pathlib import Path
 
 from ultralytics import yolo  # noqa
-from ultralytics.nn.tasks import ClassificationModel, DetectionModel, SegmentationModel, attempt_load_one_weight
+from ultralytics.nn.tasks import (ClassificationModel, DetectionModel, SegmentationModel, attempt_load_one_weight,
+                                  guess_model_task)
 from ultralytics.yolo.cfg import get_cfg
 from ultralytics.yolo.engine.exporter import Exporter
-from ultralytics.yolo.utils import DEFAULT_CFG, LOGGER, yaml_load
+from ultralytics.yolo.utils import DEFAULT_CFG, LOGGER, callbacks, yaml_load
 from ultralytics.yolo.utils.checks import check_yaml
-from ultralytics.yolo.utils.torch_utils import guess_task_from_head, smart_inference_mode
+from ultralytics.yolo.utils.torch_utils import smart_inference_mode
 
 # Map head to model, trainer, validator, and predictor classes
 MODEL_MAP = {
@@ -53,10 +54,15 @@ class YOLO:
         self.overrides = {}  # overrides for trainer object
 
         # Load or create new YOLO model
-        {'.pt': self._load, '.yaml': self._new}[Path(model).suffix](model)
+        load_methods = {'.pt': self._load, '.yaml': self._new}
+        suffix = Path(model).suffix
+        if suffix in load_methods:
+            {'.pt': self._load, '.yaml': self._new}[suffix](model)
+        else:
+            raise NotImplementedError(f"'{suffix}' model loading not implemented")
 
-    def __call__(self, source=None, stream=False, verbose=False, **kwargs):
-        return self.predict(source, stream, verbose, **kwargs)
+    def __call__(self, source=None, stream=False, **kwargs):
+        return self.predict(source, stream, **kwargs)
 
     def _new(self, cfg: str, verbose=True):
         """
@@ -68,9 +74,9 @@ class YOLO:
         """
         cfg = check_yaml(cfg)  # check YAML
         cfg_dict = yaml_load(cfg, append_filename=True)  # model dict
-        self.task = guess_task_from_head(cfg_dict["head"][-1][-2])
+        self.task = guess_model_task(cfg_dict)
         self.ModelClass, self.TrainerClass, self.ValidatorClass, self.PredictorClass = \
-            self._guess_ops_from_task(self.task)
+            self._assign_ops_from_task(self.task)
         self.model = self.ModelClass(cfg_dict, verbose=verbose)  # initialize
         self.cfg = cfg
 
@@ -87,7 +93,7 @@ class YOLO:
         self.overrides = self.model.args
         self._reset_ckpt_args(self.overrides)
         self.ModelClass, self.TrainerClass, self.ValidatorClass, self.PredictorClass = \
-            self._guess_ops_from_task(self.task)
+            self._assign_ops_from_task(self.task)
 
     def reset(self):
         """
@@ -112,7 +118,7 @@ class YOLO:
         self.model.fuse()
 
     @smart_inference_mode()
-    def predict(self, source=None, stream=False, verbose=False, **kwargs):
+    def predict(self, source=None, stream=False, **kwargs):
         """
         Perform prediction using the YOLO model.
 
@@ -120,7 +126,6 @@ class YOLO:
             source (str | int | PIL | np.ndarray): The source of the image to make predictions on.
                           Accepts all source types accepted by the YOLO model.
             stream (bool): Whether to stream the predictions or not. Defaults to False.
-            verbose (bool): Whether to print verbose information or not. Defaults to False.
             **kwargs : Additional keyword arguments passed to the predictor.
                        Check the 'configuration' section in the documentation for all available options.
 
@@ -137,7 +142,7 @@ class YOLO:
             self.predictor.setup_model(model=self.model)
         else:  # only update args if predictor is already setup
             self.predictor.args = get_cfg(self.predictor.args, overrides)
-        return self.predictor(source=source, stream=stream, verbose=verbose)
+        return self.predictor(source=source, stream=stream)
 
     @smart_inference_mode()
     def val(self, data=None, **kwargs):
@@ -212,7 +217,7 @@ class YOLO:
         """
         self.model.to(device)
 
-    def _guess_ops_from_task(self, task):
+    def _assign_ops_from_task(self, task):
         model_class, train_lit, val_lit, pred_lit = MODEL_MAP[task]
         # warning: eval is unsafe. Use with caution
         trainer_class = eval(train_lit.replace("TYPE", f"{self.type}"))
@@ -229,17 +234,16 @@ class YOLO:
         return self.model.names
 
     @staticmethod
-    def _reset_ckpt_args(args):
-        args.pop("project", None)
-        args.pop("name", None)
-        args.pop("exist_ok", None)
-        args.pop("resume", None)
-        args.pop("batch", None)
-        args.pop("epochs", None)
-        args.pop("cache", None)
-        args.pop("save_json", None)
-        args.pop("half", None)
-        args.pop("v5loader", None)
+    def add_callback(event: str, func):
+        """
+        Add callback
+        """
+        callbacks.default_callbacks[event].append(func)
 
-        # set device to '' to prevent from auto DDP usage
-        args["device"] = ''
+    @staticmethod
+    def _reset_ckpt_args(args):
+        for arg in 'verbose', 'project', 'name', 'exist_ok', 'resume', 'batch', 'epochs', 'cache', 'save_json', \
+                'half', 'v5loader':
+            args.pop(arg, None)
+
+        args["device"] = ''  # set device to '' to prevent auto-DDP usage
