@@ -5,6 +5,8 @@ import inspect
 import logging.config
 import os
 import platform
+import re
+import subprocess
 import sys
 import tempfile
 import threading
@@ -14,7 +16,6 @@ from types import SimpleNamespace
 from typing import Union
 
 import cv2
-import git
 import numpy as np
 import pandas as pd
 import torch
@@ -68,7 +69,7 @@ HELP_MSG = \
             yolo detect train data=coco128.yaml model=yolov8n.pt epochs=10 lr0=0.01
 
         - Predict a YouTube video using a pretrained segmentation model at image size 320:
-            yolo segment predict model=yolov8n-seg.pt source=https://youtu.be/Zgi9g1ksQHc imgsz=320
+            yolo segment predict model=yolov8n-seg.pt source='https://youtu.be/Zgi9g1ksQHc' imgsz=320
 
         - Val a pretrained detection model at batch-size 1 and image size 640:
             yolo detect val model=yolov8n.pt data=coco128.yaml batch=1 imgsz=640
@@ -109,10 +110,70 @@ class IterableSimpleNamespace(SimpleNamespace):
     def __str__(self):
         return '\n'.join(f"{k}={v}" for k, v in vars(self).items())
 
+    def get(self, key, default=None):
+        return getattr(self, key, default)
+
+
+def yaml_save(file='data.yaml', data=None):
+    """
+    Save YAML data to a file.
+
+    Args:
+        file (str, optional): File name. Default is 'data.yaml'.
+        data (dict, optional): Data to save in YAML format. Default is None.
+
+    Returns:
+        None: Data is saved to the specified file.
+    """
+    file = Path(file)
+    if not file.parent.exists():
+        # Create parent directories if they don't exist
+        file.parent.mkdir(parents=True, exist_ok=True)
+
+    with open(file, 'w') as f:
+        # Dump data to file in YAML format, converting Path objects to strings
+        yaml.safe_dump({k: str(v) if isinstance(v, Path) else v for k, v in data.items()}, f, sort_keys=False)
+
+
+def yaml_load(file='data.yaml', append_filename=False):
+    """
+    Load YAML data from a file.
+
+    Args:
+        file (str, optional): File name. Default is 'data.yaml'.
+        append_filename (bool): Add the YAML filename to the YAML dictionary. Default is False.
+
+    Returns:
+        dict: YAML data and file name.
+    """
+    with open(file, errors='ignore', encoding='utf-8') as f:
+        # Add YAML filename to dict and return
+        s = f.read()  # string
+        if not s.isprintable():  # remove special characters
+            s = re.sub(r'[^\x09\x0A\x0D\x20-\x7E\x85\xA0-\uD7FF\uE000-\uFFFD\U00010000-\U0010ffff]+', '', s)
+        return {**yaml.safe_load(s), 'yaml_file': str(file)} if append_filename else yaml.safe_load(s)
+
+
+def yaml_print(yaml_file: Union[str, Path, dict]) -> None:
+    """
+    Pretty prints a yaml file or a yaml-formatted dictionary.
+
+    Args:
+        yaml_file: The file path of the yaml file or a yaml-formatted dictionary.
+
+    Returns:
+        None
+    """
+    yaml_dict = yaml_load(yaml_file) if isinstance(yaml_file, (str, Path)) else yaml_file
+    dump = yaml.dump(yaml_dict, default_flow_style=False)
+    LOGGER.info(f"Printing '{colorstr('bold', 'black', yaml_file)}'\n\n{dump}")
+
 
 # Default configuration
-with open(DEFAULT_CFG_PATH, errors='ignore') as f:
-    DEFAULT_CFG_DICT = yaml.safe_load(f)
+DEFAULT_CFG_DICT = yaml_load(DEFAULT_CFG_PATH)
+for k, v in DEFAULT_CFG_DICT.items():
+    if isinstance(v, str) and v.lower() == 'none':
+        DEFAULT_CFG_DICT[k] = None
 DEFAULT_CFG_KEYS = DEFAULT_CFG_DICT.keys()
 DEFAULT_CFG = IterableSimpleNamespace(**DEFAULT_CFG_DICT)
 
@@ -124,8 +185,7 @@ def is_colab():
     Returns:
         bool: True if running inside a Colab notebook, False otherwise.
     """
-    # Check if the google.colab module is present in sys.modules
-    return 'google.colab' in sys.modules
+    return 'COLAB_RELEASE_TAG' in os.environ or 'COLAB_BACKEND_VERSION' in os.environ
 
 
 def is_kaggle():
@@ -138,7 +198,7 @@ def is_kaggle():
     return os.environ.get('PWD') == '/kaggle/working' and os.environ.get('KAGGLE_URL_BASE') == 'https://www.kaggle.com'
 
 
-def is_jupyter_notebook():
+def is_jupyter():
     """
     Check if the current script is running inside a Jupyter Notebook.
     Verified on Colab, Jupyterlab, Kaggle, Paperspace.
@@ -146,8 +206,6 @@ def is_jupyter_notebook():
     Returns:
         bool: True if running inside a Jupyter Notebook, False otherwise.
     """
-    # Check if the get_ipython function exists
-    # (it does not exist when running as a standalone script)
     try:
         from IPython import get_ipython
         return get_ipython() is not None
@@ -170,21 +228,6 @@ def is_docker() -> bool:
         return False
 
 
-def is_git_directory() -> bool:
-    """
-    Check if the current working directory is inside a git repository.
-
-    Returns:
-        bool: True if the current working directory is inside a git repository, False otherwise.
-    """
-    try:
-        git.Repo(search_parent_directories=True)
-        # subprocess.run(["git", "rev-parse", "--git-dir"], capture_output=True, check=True)  # CLI alternative
-        return True
-    except git.exc.InvalidGitRepositoryError:  # subprocess.CalledProcessError:
-        return False
-
-
 def is_pip_package(filepath: str = __name__) -> bool:
     """
     Determines if the file at the given filepath is part of a pip package.
@@ -224,8 +267,10 @@ def is_dir_writeable(dir_path: Union[str, Path]) -> bool:
 
 def is_pytest_running():
     """
-    Returns a boolean indicating if pytest is currently running or not
-    :return: True if pytest is running, False otherwise
+    Determines whether pytest is currently running or not.
+
+    Returns:
+        (bool): True if pytest is running, False otherwise.
     """
     try:
         import sys
@@ -234,17 +279,53 @@ def is_pytest_running():
         return False
 
 
-def get_git_root_dir():
+def is_github_actions_ci() -> bool:
+    """
+    Determine if the current environment is a GitHub Actions CI Python runner.
+
+    Returns:
+        (bool): True if the current environment is a GitHub Actions CI Python runner, False otherwise.
+    """
+    return 'GITHUB_ACTIONS' in os.environ and 'RUNNER_OS' in os.environ and 'RUNNER_TOOL_CACHE' in os.environ
+
+
+def is_git_dir():
+    """
+    Determines whether the current file is part of a git repository.
+    If the current file is not part of a git repository, returns None.
+
+    Returns:
+        (bool): True if current file is part of a git repository.
+    """
+    return get_git_dir() is not None
+
+
+def get_git_dir():
     """
     Determines whether the current file is part of a git repository and if so, returns the repository root directory.
     If the current file is not part of a git repository, returns None.
+
+    Returns:
+        (Path) or (None): Git root directory if found or None if not found.
     """
-    try:
-        # output = subprocess.run(["git", "rev-parse", "--git-dir"], capture_output=True, check=True)
-        # return Path(output.stdout.strip().decode('utf-8')).parent.resolve()  # CLI alternative
-        return Path(git.Repo(search_parent_directories=True).working_tree_dir)
-    except git.exc.InvalidGitRepositoryError:  # (subprocess.CalledProcessError, FileNotFoundError):
-        return None
+    for d in Path(__file__).parents:
+        if (d / '.git').is_dir():
+            return d
+    return None  # no .git dir found
+
+
+def get_git_origin_url():
+    """
+    Retrieves the origin URL of a git repository.
+
+    Returns:
+        (str) or (None): The origin URL of the git repository.
+    """
+    if is_git_dir():
+        with contextlib.suppress(subprocess.CalledProcessError):
+            origin = subprocess.check_output(["git", "config", "--get", "remote.origin.url"])
+            return origin.decode().strip()
+    return None  # if not git dir or on error
 
 
 def get_default_args(func):
@@ -316,7 +397,7 @@ def colorstr(*input):
         "bright_white": "\033[97m",
         "end": "\033[0m",  # misc
         "bold": "\033[1m",
-        "underline": "\033[4m",}
+        "underline": "\033[4m"}
     return "".join(colors[x] for x in args) + f"{string}" + colors["end"]
 
 
@@ -334,12 +415,12 @@ def set_logging(name=LOGGING_NAME, verbose=True):
             name: {
                 "class": "logging.StreamHandler",
                 "formatter": name,
-                "level": level,}},
+                "level": level}},
         "loggers": {
             name: {
                 "level": level,
                 "handlers": [name],
-                "propagate": False,}}})
+                "propagate": False}}})
 
 
 class TryExcept(contextlib.ContextDecorator):
@@ -367,72 +448,35 @@ def threaded(func):
     return wrapper
 
 
-def yaml_save(file='data.yaml', data=None):
-    """
-    Save YAML data to a file.
-
-    Args:
-        file (str, optional): File name. Default is 'data.yaml'.
-        data (dict, optional): Data to save in YAML format. Default is None.
-
-    Returns:
-        None: Data is saved to the specified file.
-    """
-    file = Path(file)
-    if not file.parent.exists():
-        # Create parent directories if they don't exist
-        file.parent.mkdir(parents=True, exist_ok=True)
-
-    with open(file, 'w') as f:
-        # Dump data to file in YAML format, converting Path objects to strings
-        yaml.safe_dump({k: str(v) if isinstance(v, Path) else v for k, v in data.items()}, f, sort_keys=False)
-
-
-def yaml_load(file='data.yaml', append_filename=False):
-    """
-    Load YAML data from a file.
-
-    Args:
-        file (str, optional): File name. Default is 'data.yaml'.
-        append_filename (bool): Add the YAML filename to the YAML dictionary. Default is False.
-
-    Returns:
-        dict: YAML data and file name.
-    """
-    with open(file, errors='ignore') as f:
-        # Add YAML filename to dict and return
-        return {**yaml.safe_load(f), 'yaml_file': str(file)} if append_filename else yaml.safe_load(f)
-
-
-def yaml_print(yaml_file: Union[str, Path, dict]) -> None:
-    """
-    Pretty prints a yaml file or a yaml-formatted dictionary.
-
-    Args:
-        yaml_file: The file path of the yaml file or a yaml-formatted dictionary.
-
-    Returns:
-        None
-    """
-    yaml_dict = yaml_load(yaml_file) if isinstance(yaml_file, (str, Path)) else yaml_file
-    dump = yaml.dump(yaml_dict, default_flow_style=False)
-    LOGGER.info(f"Printing '{colorstr('bold', 'black', yaml_file)}'\n\n{dump}")
-
-
-def set_sentry(dsn="https://1f331c322109416595df20a91f4005d3@o4504521589325824.ingest.sentry.io/4504521592406016"):
+def set_sentry():
     """
     Initialize the Sentry SDK for error tracking and reporting if pytest is not currently running.
     """
-    if dsn and not is_pytest_running():
+
+    def before_send(event, hint):
+        env = 'Colab' if is_colab() else 'Kaggle' if is_kaggle() else 'Jupyter' if is_jupyter() else \
+            'Docker' if is_docker() else platform.system()
+        event['tags'] = {
+            "sys_argv": sys.argv[0],
+            "sys_argv_name": Path(sys.argv[0]).name,
+            "install": 'git' if is_git_dir() else 'pip' if is_pip_package() else 'other',
+            "os": env}
+        return event
+
+    if SETTINGS['sync'] and \
+            not is_pytest_running() and \
+            not is_github_actions_ci() and \
+            (is_pip_package() or get_git_origin_url() == "https://github.com/ultralytics/ultralytics.git"):
         import sentry_sdk  # noqa
 
         import ultralytics
         sentry_sdk.init(
-            dsn=dsn,
+            dsn="https://1f331c322109416595df20a91f4005d3@o4504521589325824.ingest.sentry.io/4504521592406016",
             debug=False,
             traces_sample_rate=1.0,
             release=ultralytics.__version__,
             environment='production',  # 'dev' or 'production'
+            before_send=before_send,
             ignore_errors=[KeyboardInterrupt])
 
 
@@ -450,9 +494,9 @@ def get_settings(file=USER_CONFIG_DIR / 'settings.yaml', version='0.0.1'):
     from ultralytics.yolo.utils.checks import check_version
     from ultralytics.yolo.utils.torch_utils import torch_distributed_zero_first
 
-    is_git = is_git_directory()  # True if ultralytics installed via git
-    root = get_git_root_dir() if is_git else Path()
-    datasets_root = (root.parent if (is_git and is_dir_writeable(root.parent)) else root).resolve()
+    git_dir = get_git_dir()
+    root = git_dir or Path()
+    datasets_root = (root.parent if git_dir and is_dir_writeable(root.parent) else root).resolve()
     defaults = {
         'datasets_dir': str(datasets_root / 'datasets'),  # default datasets directory.
         'weights_dir': str(root / 'weights'),  # default weights directory.
@@ -464,13 +508,13 @@ def get_settings(file=USER_CONFIG_DIR / 'settings.yaml', version='0.0.1'):
     with torch_distributed_zero_first(RANK):
         if not file.exists():
             yaml_save(file, defaults)
-
         settings = yaml_load(file)
 
         # Check that settings keys and types match defaults
-        correct = settings.keys() == defaults.keys() \
-                  and all(type(a) == type(b) for a, b in zip(settings.values(), defaults.values())) \
-                  and check_version(settings['settings_version'], version)
+        correct = \
+            settings.keys() == defaults.keys() \
+            and all(type(a) == type(b) for a, b in zip(settings.values(), defaults.values())) \
+            and check_version(settings['settings_version'], version)
         if not correct:
             LOGGER.warning('WARNING ⚠️ Ultralytics settings reset to defaults. '
                            '\nThis is normal and may be due to a recent ultralytics package update, '
@@ -491,7 +535,7 @@ def set_settings(kwargs, file=USER_CONFIG_DIR / 'settings.yaml'):
     yaml_save(file, SETTINGS)
 
 
-# Run below code on utils init -----------------------------------------------------------------------------------------
+# Run below code on yolo/utils init ------------------------------------------------------------------------------------
 
 # Set logger
 set_logging(LOGGING_NAME)  # run before defining LOGGER
